/* eslint-env jest */
import React from 'react';
import Enzyme, { shallow } from 'enzyme';
import FormPta from '../index';
import Adapter from 'enzyme-adapter-react-16';
Enzyme.configure({
  adapter: new Adapter()
});
describe('<FormPta />', function () {
  describe('submit listeners', function () {
    var component, wrapper;
    var onSubmitSpy = jest.fn();
    var onErrorSpy = jest.fn();
    beforeEach(function () {
      // Given
      component = React.createElement(FormPta, {
        onSubmit: onSubmitSpy,
        onError: onErrorSpy,
        formUrl: "anyUrl"
      });
      wrapper = shallow(component);
    });
    afterEach(function () {
      onSubmitSpy.mockRestore();
      onErrorSpy.mockRestore();
      wrapper.unmount();
    });
    it('should call onSubmit callback when SUBMIT message is listened from PTA', function (done) {
      var EVENT_TYPE = 'SUBMIT_FORM_SUCCEEDED';

      var listener = function listener(e) {
        // Then
        expect(onSubmitSpy.mock.calls.length).toEqual(1);
        done();
      };

      window.addEventListener('message', listener, {
        once: true
      }); // When

      window.postMessage({
        type: EVENT_TYPE
      }, '*');
    });
    it('should call onError callback when SUBMIT message is listened from PTA', function (done) {
      var EVENT_TYPE = 'SUBMIT_FORM_FAILED';

      var listener = function listener(e) {
        // Then
        expect(onErrorSpy.mock.calls.length).toEqual(1);
        done();
      };

      window.addEventListener('message', listener, {
        once: true
      }); // When

      window.postMessage({
        type: EVENT_TYPE
      }, '*');
    });
    it('should NOT call onSubmit callback when NO SUBMIT message is listened from PTA', function (done) {
      var NO_SUBMIT_TYPE = 'NOSUBMIT';

      var listener = function listener(e) {
        // Then
        expect(onSubmitSpy.mock.calls.length).toEqual(0);
        expect(onErrorSpy.mock.calls.length).toEqual(0);
        done();
      };

      window.addEventListener('message', listener, {
        once: true
      }); // When

      window.postMessage({
        type: NO_SUBMIT_TYPE
      }, '*');
    });
    it('shoul call eventListeners when event emmited type matches', function (done) {
      var listener = function listener(e) {
        // Then
        expect(onSubmitSpy.mock.calls.length).toEqual(0);
        expect(onErrorSpy.mock.calls.length).toEqual(0);
        done();
      };

      window.addEventListener('message', listener, {
        once: true
      }); // When

      window.postMessage({
        type: 'ANY_TYPE'
      }, '*');
    });
  });
  describe('generic listeners', function () {
    var component, wrapper;
    var listener1Spy = jest.fn();
    var listener2Spy = jest.fn();
    var listener3Spy = jest.fn();
    var EVENT_TYPE_1 = 'EVENT_TYPE_1';
    var EVENT_TYPE_2 = 'EVENT_TYPE_2';
    var EVENT_TYPE_3 = 'EVENT_TYPE_3';
    var NOT_MATCHING_EVENT_TYPE = 'NOT_MATCHING_EVENT_TYPE';
    var eventListeners = [{
      eventNames: [EVENT_TYPE_1, EVENT_TYPE_2],
      listener: listener1Spy
    }, {
      eventNames: [EVENT_TYPE_1],
      listener: listener2Spy
    }, {
      eventNames: [EVENT_TYPE_3],
      listener: listener3Spy
    }];
    beforeEach(function () {
      // Given
      component = React.createElement(FormPta, {
        eventListeners: eventListeners,
        formUrl: "anyUrl"
      });
      wrapper = shallow(component);
    });
    afterEach(function () {
      listener1Spy.mockRestore();
      listener2Spy.mockRestore();
      wrapper.unmount();
    });
    it('shoul call eventListeners when event emmited type matches many listener', function (done) {
      var listener = function listener(e) {
        // Then
        expect(listener1Spy.mock.calls.length).toEqual(1);
        expect(listener2Spy.mock.calls.length).toEqual(1);
        expect(listener3Spy.mock.calls.length).toEqual(0);
        done();
      };

      window.addEventListener('message', listener, {
        once: true
      }); // When

      window.postMessage({
        type: EVENT_TYPE_1
      }, '*');
    });
    it('shoul call eventListeners when event emmited type matches one listener', function (done) {
      var listener = function listener(e) {
        // Then
        expect(listener1Spy.mock.calls.length).toEqual(1);
        expect(listener2Spy.mock.calls.length).toEqual(0);
        expect(listener3Spy.mock.calls.length).toEqual(0);
        done();
      };

      window.addEventListener('message', listener, {
        once: true
      }); // When

      window.postMessage({
        type: EVENT_TYPE_2
      }, '*');
    });
    it('shoul not call eventListeners when event emmited type does not match any listener', function (done) {
      var listener = function listener(e) {
        // Then
        expect(listener1Spy.mock.calls.length).toEqual(0);
        expect(listener2Spy.mock.calls.length).toEqual(0);
        expect(listener3Spy.mock.calls.length).toEqual(0);
        done();
      };

      window.addEventListener('message', listener, {
        once: true
      }); // When

      window.postMessage({
        type: NOT_MATCHING_EVENT_TYPE
      }, '*');
    });
  });
});